@@ -18,7 +18,7 @@
 #'   'tls+tcp://10.75.32.70:5555'.
 #' @param dispatcher \[default TRUE\] logical value, which should be set to
 #'   TRUE if using dispatcher and FALSE otherwise.
-#' @param ... reserved but not currently used.
+#' @param ... reserved, but not currently used.
 #' @param asyncdial \[default FALSE\] whether to perform dials asynchronously.
 #'   The default FALSE will error if a connection is not immediately possible
 #'   (for instance if [daemons()] has yet to be called on the host, or the
@@ -43,15 +43,16 @@
 #'   limit) i.e. the minimum amount of real time elapsed before exiting.
 #' @param maxtasks \[default Inf\] integer maximum number of tasks to execute
 #'   (task limit) before exiting.
-#' @param id \[default NULL\] (optional) integer daemon ID provided to
-#'   dispatcher to track connection status. Causes [status()] to report this ID
-#'   under `$events` when the daemon connects and disconnects.
-#' @param tls \[default NULL\] required for secure TLS connections over
+#' @param tlscert \[default NULL\] required for secure TLS connections over
 #'   'tls+tcp://'. **Either** the character path to a file containing X.509
 #'   certificate(s) in PEM format, comprising the certificate authority
 #'   certificate chain starting with the TLS certificate and ending with the CA
 #'   certificate, **or** a length 2 character vector comprising \[i\] the
 #'   certificate authority certificate chain and \[ii\] the empty string `""`.
+#' @param tls deprecated, please use `tlscert` instead.
+#' @param id \[default NULL\] (optional) integer daemon ID provided to
+#'   dispatcher to track connection status. Causes [status()] to report this ID
+#'   under `$events` when the daemon connects and disconnects.
 #' @param rs \[default NULL\] the initial value of .Random.seed. This is set
 #'   automatically using L'Ecuyer-CMRG RNG streams generated by the host process
 #'   if applicable, and should not be independently supplied.
@@ -75,8 +76,8 @@
 #' Setting to FALSE allows the daemon to persist indefinitely even when there is
 #' no longer a socket connection. This allows a host session to end and a new
 #' session to connect at the URL where the daemon is dialled in. Daemons must be
-#' terminated with `daemons(NULL)` in this case, which sends explicit exit
-#' signals to all connected daemons.
+#' terminated with `daemons(NULL)` in this case instead of `daemons(0)`. This
+#' sends explicit exit signals to all connected daemons.
 #'
 #' @export
 #'
@@ -91,36 +92,37 @@ daemon <- function(
   idletime = Inf,
   walltime = Inf,
   maxtasks = Inf,
+  tlscert = NULL,
+  tls = tlscert,
   id = NULL,
-  tls = NULL,
   rs = NULL
 ) {
   cv <- cv()
   sock <- socket(if (dispatcher) "poly" else "rep")
-  on.exit(reap(sock))
+  on.exit({
+    reap(sock)
+    `[[<-`(., "sock", NULL)
+  })
+  `[[<-`(., "sock", sock)
   pipe_notify(sock, cv, remove = TRUE, flag = flag_value_auto(autoexit))
   if (length(tls)) tls <- tls_config(client = tls)
   dial_sync_socket(sock, url, autostart = asyncdial || NA, tls = tls)
 
-  `[[<-`(., "sock", sock)
-  on.exit(`[[<-`(., "sock", NULL), add = TRUE)
   if (!output) {
     devnull <- file(nullfile(), open = "w", blocking = FALSE)
     sink(file = devnull)
     sink(file = devnull, type = "message")
-    on.exit(
-      {
-        sink(type = "message")
-        sink()
-        close(devnull)
-      },
-      add = TRUE
-    )
   }
   xc <- 0L
   task <- 1L
   timeout <- if (idletime > walltime) walltime else if (is.finite(idletime)) idletime
   maxtime <- if (is.finite(walltime)) mclock() + walltime else FALSE
+  if (otel_tracing) {
+    spn <- otel::start_local_active_span(
+      "mirai::daemon",
+      attributes = otel::as_attributes(list(url = url))
+    )
+  }
 
   if (dispatcher) {
     aio <- recv_aio(sock, mode = 1L, cv = cv)
@@ -139,18 +141,15 @@ daemon <- function(
         xc <- 1L
         break
       }
-      cancel <- recv_aio(sock, mode = 8L, cv = NA)
-      data <- eval_mirai(m)
-      stop_aio(cancel)
-      { task >= maxtasks || maxtime && mclock() >= maxtime } && {
+      (task >= maxtasks || maxtime && mclock() >= maxtime) && {
         .mark()
-        send(sock, data, mode = 1L, block = TRUE)
+        send(sock, eval_mirai_with_cancel(m, sock), mode = 1L, block = TRUE)
         aio <- recv_aio(sock, mode = 8L, cv = cv)
         xc <- 2L + (task >= maxtasks)
         wait(cv)
         break
       }
-      send(sock, data, mode = 1L, block = TRUE)
+      send(sock, eval_mirai_with_cancel(m, sock), mode = 1L, block = TRUE)
       if (cleanup) do_cleanup()
       task <- task + 1L
     }
@@ -166,10 +165,9 @@ daemon <- function(
         xc <- 1L
         break
       }
-      data <- eval_mirai(m)
-      send(ctx, data, mode = 1L, block = TRUE)
+      send(ctx, eval_mirai(m), mode = 1L, block = TRUE)
       if (cleanup) do_cleanup()
-      { task >= maxtasks || maxtime && mclock() >= maxtime } && {
+      (task >= maxtasks || maxtime && mclock() >= maxtime) && {
         xc <- 2L + (task >= maxtasks)
         break
       }
@@ -177,6 +175,11 @@ daemon <- function(
     }
   }
 
+  if (!output) {
+    sink(type = "message")
+    sink()
+    close.connection(devnull)
+  }
   invisible(xc)
 }
 
@@ -198,25 +201,36 @@ daemon <- function(
   pipe_notify(sock, cv, remove = TRUE, flag = flag_value())
   dial(sock, url = url, autostart = NA, fail = 2L)
   `[[<-`(., "sock", sock)
-  data <- eval_mirai(recv(sock, mode = 1L, block = TRUE))
-  send(sock, data, mode = 1L, block = TRUE) || until(cv, .limit_short)
+  m <- recv(sock, mode = 1L, block = TRUE)
+  send(sock, eval_mirai(m), mode = 1L, block = TRUE) || until(cv, .limit_short)
 }
 
 # internals --------------------------------------------------------------------
 
-handle_mirai_error <- function(cnd)
+handle_mirai_error <- function(cnd) {
+  if (otel_tracing) otel::get_active_span()$set_status("error", "miraiError")
   invokeRestart("mirai_error", cnd, sys.calls())
+}
 
-handle_mirai_interrupt <- function(cnd) invokeRestart("mirai_interrupt")
+handle_mirai_interrupt <- function(cnd) {
+  if (otel_tracing) otel::get_active_span()$set_status("error", "miraiInterrupt")
+  invokeRestart("mirai_interrupt")
+}
 
 eval_mirai <- function(._mirai_.) {
   withRestarts(
     withCallingHandlers(
       {
-        on.exit(.interrupt(FALSE))
-        .interrupt()
-        list2env(._mirai_.[["._mirai_globals_."]], envir = .GlobalEnv)
-        eval(._mirai_.[[".expr"]], envir = ._mirai_., enclos = .GlobalEnv)
+        list2env(._mirai_.[["._globals_."]], envir = .GlobalEnv)
+        if (otel_tracing && length(._mirai_.[["._otel_."]])) {
+          prtctx <- otel::extract_http_context(._mirai_.[["._otel_."]])
+          spn <- otel::start_local_active_span(
+            "mirai::daemon->eval",
+            links = list(daemon = otel::get_active_span()),
+            options = list(kind = "server", parent = prtctx)
+          )
+        }
+        eval(._mirai_.[["._expr_."]], envir = ._mirai_., enclos = .GlobalEnv)
       },
       error = handle_mirai_error,
       interrupt = handle_mirai_interrupt
@@ -226,6 +240,12 @@ eval_mirai <- function(._mirai_.) {
   )
 }
 
+eval_mirai_with_cancel <- function(._mirai_., sock) {
+  cancel <- recv_aio(sock, mode = 8L, cv = substitute())
+  on.exit(stop_aio(cancel))
+  eval_mirai(._mirai_.)
+}
+
 dial_sync_socket <- function(sock, url, autostart = NA, tls = NULL) {
   cv <- cv()
   pipe_notify(sock, cv, add = TRUE)
@@ -242,10 +262,11 @@ do_cleanup <- function() {
   options(.[["op"]])
 }
 
-snapshot <- function()
-  `[[<-`(`[[<-`(`[[<-`(., "op", .Options), "se", search()), "vars", names(.GlobalEnv))
+snapshot <- function() `[[<-`(`[[<-`(`[[<-`(., "op", .Options), "se", search()), "vars", names(.GlobalEnv))
 
-flag_value_auto <- function(autoexit)
-  { isFALSE(autoexit) || isNamespace(topenv(parent.frame(), NULL)) } && return(autoexit) || is.na(autoexit) || isNamespaceLoaded("covr") || return(tools::SIGTERM)
+flag_value_auto <- function(autoexit) {
+  (isFALSE(autoexit) || isNamespace(topenv(parent.frame(), NULL))) && return(autoexit) ||
+    is.na(autoexit) || isNamespaceLoaded("covr") || return(tools::SIGTERM)
+}
 
 flag_value <- function() isNamespaceLoaded("covr") || return(tools::SIGTERM)
