@@ -21,15 +21,22 @@
 #'  This may be overriden, if desired, by specifying 'url' in the [daemons()]
 #'  interface and launching daemons using [launch_local()].
 #'
+#' @section OpenTelemetry:
+#'
+#' mirai provides comprehensive OpenTelemetry tracing support for observing
+#' asynchronous operations and distributed computation. Please refer to the
+#' OpenTelemetry vignette for further details:
+#' `vignette("v05-opentelemetry", package = "mirai")`
+#'
 #' @section Reference Manual:
 #'
 #' `vignette("mirai", package = "mirai")`
 #'
 #' @importFrom nanonext .advance call_aio call_aio_ collect_aio collect_aio_
-#'   .context cv cv_signal cv_value dial .interrupt ip_addr
-#'   is_error_value .keep listen .mark mclock monitor msleep nng_error opt opt<-
-#'   parse_url pipe_id pipe_notify random .read_header .read_marker read_monitor
-#'   reap recv recv_aio request send serial_config socket stat stop_aio
+#'   .context cv cv_signal cv_value dial ip_addr is_error_value .keep listen
+#'   .mark mclock monitor msleep nng_error opt opt<- parse_url pipe_id
+#'   pipe_notify random .read_header .read_marker read_monitor reap recv
+#'   recv_aio request send serial_config socket stat stop_aio stop_request
 #'   tls_config unresolved .unresolved until wait write_cert
 #'
 "_PACKAGE"
@@ -38,6 +45,7 @@
 # tested implicitly
 
 .onLoad <- function(libname, pkgname) {
+  otel_tracing <<- requireNamespace("otel", quietly = TRUE) && otel::is_tracing_enabled()
   switch(
     Sys.info()[["sysname"]],
     Linux = {
@@ -69,19 +77,15 @@
   list(
     arglen = "`n` must equal the length of `args`, or either must be 1",
     cluster_inactive = "cluster is no longer active",
-    daemons_set = "daemons already set for `%s` compute profile",
     daemons_unset = "daemons must be set to use launchers",
-    dispatcher_args = "`dispatcher` should be either TRUE or FALSE",
     dot_required = "`.` must be an element of the character vector(s) supplied to `args`",
     function_required = "`.f` must be of type function, not %s",
     localhost = "SSH tunnelling requires daemons `url` hostname to be `127.0.0.1`",
     missing_expression = "missing expression, perhaps wrap in {}?",
-    missing_url = "`n` must be 1 or greater, or else `url` must be supplied",
     named_args = "all items in `.args` must be named, unless supplying an environment",
     named_dots = "all `...` arguments must be named, unless supplying an environment",
     n_one = "`n` must be 1 or greater",
     n_zero = "the number of daemons must be zero or greater",
-    not_found = "compute profile `%s` not found",
     numeric_n = "`n` must be numeric, did you mean to provide `url`?",
     sync_daemons = "mirai: initial sync with daemon(s) [%d secs elapsed]",
     sync_dispatcher = "mirai: initial sync with dispatcher [%d secs elapsed]",
@@ -89,3 +93,6 @@
   ),
   hash = TRUE
 )
+
+otel_tracing <- FALSE
+otel_tracer_name <- "org.r-lib.mirai"
