@@ -46,34 +46,25 @@ as.promise.mirai <- function(x) {
   promise <- .subset2(x, "promise")
 
   if (is.null(promise)) {
-    promise <- if (unresolved(x)) {
-      promises::promise(
-        function(resolve, reject) .keep(x, environment())
-      )$then(
-        onFulfilled = function(value, .visible) {
-          is_error_value(value) && !is_mirai_interrupt(value) &&
-            stop(if (is_mirai_error(value)) value else nng_error(value))
-          value
-        }
-      )
-    } else {
-      value <- .subset2(x, "value")
-      promises::promise(
-        function(resolve, reject)
-          resolve({
-            is_error_value(value) && !is_mirai_interrupt(value) &&
-              stop(if (is_mirai_error(value)) value else nng_error(value))
-            value
-          })
-      )
-    }
-
+    promise <- promises::promise(
+      function(resolve, reject) {
+        if (unresolved(x)) .keep(x, environment()) else resolve(.subset2(x, "value"))
+      }
+    )$then(
+      onFulfilled = handle_fulfilled
+    )
     `[[<-`(x, "promise", promise)
   }
 
   promise
 }
 
+handle_fulfilled <- function(value, .visible) {
+  is_error_value(value) && !is_mirai_interrupt(value) &&
+    stop(if (is_mirai_error(value)) value else nng_error(value))
+  value
+}
+
 #' Make mirai_map Promise
 #'
 #' Creates a 'promise' from a 'mirai_map'.
@@ -111,8 +102,9 @@ as.promise.mirai <- function(x) {
 as.promise.mirai_map <- function(x) {
   promise <- attr(x, "promise")
 
-  if (is.null(promise))
-    attr(x, "promise") <- promise <- promises::promise_all(.list = x)
+  if (is.null(promise)) {
+    attr(x, "promise") <- promises::promise_all(.list = x) -> promise
+  }
 
   promise
 }
