@@ -1,7 +1,9 @@
-# minitest - a minimal testing framework v0.0.2 --------------------------------
+# minitest - a minimal testing framework v0.0.4 --------------------------------
 test_library <- function(package) library(package = package, character.only = TRUE)
 test_true <- function(x) invisible(isTRUE(x) || {print(x); stop("the above was returned instead of TRUE")})
+test_false <- function(x) invisible(isFALSE(x) || {print(x); stop("the above was returned instead of FALSE")})
 test_null <- function(x) invisible(is.null(x) || {print(x); stop("the above was returned instead of NULL")})
+test_notnull <- function(x) invisible(!is.null(x) || stop("returns NULL when expected to be not NULL"))
 test_zero <- function(x) invisible(x == 0L || {print(x); stop("the above was returned instead of 0L")})
 test_type <- function(type, x) invisible(typeof(x) == type || {stop("object of type '", typeof(x), "' was returned instead of '", type, "'")})
 test_class <- function(class, x) invisible(inherits(x, class) || {stop("object of class '", paste(class(x), collapse = ", "), "' was returned instead of '", class, "'")})
@@ -14,10 +16,11 @@ test_error <- function(x, containing = "") invisible(inherits(x <- tryCatch(x, e
 test_library("mirai")
 connection <- !is_error_value(collect_mirai(mirai(TRUE, .timeout = 2000L)))
 # core tests
+test_null(info())
 test_type("list", status())
 test_zero(status()[["connections"]])
 test_zero(status()[["daemons"]])
-test_zero(daemons(0L))
+test_false(daemons(0L))
 test_error(mirai(), "missing expression, perhaps wrap in {}?")
 test_error(mirai(a, 1), "all `...` arguments must be named")
 test_error(mirai(a, .args = list(1)), "all items in `.args` must be named")
@@ -25,14 +28,14 @@ test_error(mirai_map(1:2, identity))
 test_error(daemons(url = "URL"), "Invalid argument")
 test_error(daemons(-1), "zero or greater")
 test_error(daemons(raw(0L)), "must be numeric")
-test_error(daemons(1, dispatcher = 1L), "should be either TRUE or FALSE")
-test_error(daemons(url = local_url(), dispatcher = 0), "should be either TRUE or FALSE")
-test_error(dispatcher(client = "URL"), "must be 1 or greater")
+test_error(daemons(1, dispatcher = ""))
+test_error(daemons(url = local_url(), dispatcher = NA))
 test_error(daemon("URL"), "Invalid argument")
 test_error(launch_local(1L), "daemons must be set")
-test_true(!daemons_set())
+test_false(daemons_set())
 test_error(require_daemons())
-test_error(mirai:::stop_d(), "No daemons set")
+test_error(mirai:::stop_d(NULL), "No daemons set.")
+test_error(mirai:::stop_d("default"), "No daemons set for the 'default' compute profile.")
 test_type("character", host_url())
 test_type("character", names(host_url()))
 test_true(all(startsWith(host_url(tls = TRUE), "tls")))
@@ -46,12 +49,11 @@ test_true(is_mirai_interrupt(r <- mirai:::mk_interrupt_error()))
 test_print(r)
 test_true(is_mirai_error(r <- `class<-`("Error in: testing\n", c("miraiError", "errorValue", "try-error"))))
 test_print(r)
-test_null(r$stack.trace)
-test_true(!on_daemon())
+test_false(on_daemon())
 test_equal(mirai:::.DollarNames.miraiError(r, "c"), "class")
 test_true(mirai:::is.promising.mirai())
 test_true(mirai:::is.promising.mirai_map())
-test_error(everywhere({}), "not found")
+test_error(everywhere({}))
 test_null(nextstream())
 test_null(nextget("pid"))
 for (i in 0:4)
@@ -69,28 +71,29 @@ connection && {
     q / m
   }, m = 2L, .args = environment(), .timeout = 2000L)
   test_identical(call_mirai(m), m)
+  test_type("integer", attr(m, "id"))
   if (!is_error_value(m$data)) test_equal(m$data, 3L)
   Sys.sleep(1L)
   `lang obj` <- quote(m + n + 2L)
   args <- c(m = 2L, n = 4L)
   m <- mirai(.expr = `lang obj`, .args = args, .timeout = 2000L)
   if (!is_error_value(call_mirai(m)$data)) test_equal(m$data, 8L)
-  test_true(!stop_mirai(m))
+  test_false(stop_mirai(m))
   Sys.sleep(1L)
   m <- mirai(mirai::on_daemon(), .timeout = 2000L)
   if (!is_error_value(m[])) test_true(m[])
   Sys.sleep(1L)
-  test_equal(1L, d <- daemons(1L, dispatcher = FALSE, asyncdial = FALSE, seed = 1546L))
+  test_true(d <- daemons(1L, dispatcher = FALSE, asyncdial = FALSE, seed = 1546L))
   test_print(d)
-  test_error(daemons(1L), "daemons already set")
   test_true(daemons_set())
-  test_true(require_daemons())
+  test_true(require_daemons(parent.frame()))
   me <- mirai(mirai::mirai(), .timeout = 2000L)[]
-  if (!is_mirai_error(me)) test_true(is_error_value(me))
+  test_notnull(me)
   if (is_mirai_error(me)) test_type("list", me$stack.trace)
   if (is_mirai_error(me)) test_true(length(me$stack.trace) >= 2L)
   if (is_mirai_error(me)) test_true(all(as.logical(lapply(me$stack.trace, is.language))))
-  test_true(!is_mirai_interrupt(me))
+  if (is_mirai_error(me)) test_type("character", me$condition.class)
+  test_false(is_mirai_interrupt(me))
   test_class("errorValue", me)
   test_print(me)
   m <- mirai(mirai::on_daemon(), .timeout = 2000L)
@@ -98,9 +101,10 @@ connection && {
   df <- data.frame(a = 1, b = 2, .Random.seed = 0)
   dm <- mirai(as.matrix(df), .args = list(df = df), .timeout = 2000L)
   test_true(is_mirai(call_mirai(dm)))
-  test_true(!unresolved(dm))
+  test_false(unresolved(dm))
   if (!is_error_value(dm$data)) test_class("matrix", dm$data)
   test_print(dm)
+  test_type("integer", info())
   test_type("integer", status()[["connections"]])
   test_type("character", status()[["daemons"]])
   test_type("character", mlc <- launch_remote())
@@ -108,37 +112,44 @@ connection && {
   test_print(mlc)
   test_error(launch_remote(1L, remote = remote_config(command = "echo", args = "invalid")), "must be an element")
   test_error(launch_remote(3L, remote = remote_config(command = "echo", args = list(c("test", "."), c("test", ".")))), "must equal the length")
-  test_zero(daemons(0L))
+  test_false(daemons(0L))
   Sys.sleep(1L)
-  test_equal(1L, daemons(1L, dispatcher = FALSE, maxtasks = 10L, walltime = 10000L, idletime = 20000L, cleanup = FALSE, output = TRUE, .compute = "new"))
+  test_true(daemons(1L, dispatcher = FALSE, maxtasks = 10L, walltime = 10000L, idletime = 20000L, cleanup = FALSE, output = TRUE, .compute = "new"))
   test_type("character", nextget("url", .compute = "new"))
   test_type("integer", nextstream(.compute = "new"))
+  test_true(require_daemons("new", call = environment()))
   Sys.sleep(1.5)
-  test_type("list", everywhere({}, as.environment(df), .compute = "new"))
-  mn <- mirai("test1", .compute = "new")
-  mp <- mirai(b + 1, .compute = "new")
+  run_checks_on <- function(.compute, check) {
+    local_daemons(.compute)
+    check
+  }
+  run_checks_on("new", test_type("list", everywhere({}, as.environment(df))))
+  test_zero(status()$connections)
+  with_daemons("new", {
+    mn <- mirai("test1")
+    mp <- mirai(b + 1)
+  })
   Sys.sleep(1L)
   if (!unresolved(mn$data)) test_equal(mn$data, "test1")
   if (!unresolved(mp$data)) test_equal(mp$data, 3)
   Sys.sleep(1L)
   test_type("integer", status(.compute = "new")[["connections"]])
   test_error(mirai_map(1:2, "a function", .compute = "new"), "must be of type function, not character")
-  test_error(daemons(url = local_url(), .compute = "new"), "daemons already set")
-  test_zero(daemons(0L, .compute = "new"))
+  test_false(daemons(0L, .compute = "new"))
 }
 # additional daemons tests
 connection && {
   Sys.sleep(1L)
-  test_zero(daemons(url = value <- local_url(), dispatcher = FALSE))
+  test_true(daemons(url = value <- local_url(), dispatcher = FALSE))
   test_identical(status()$daemons, value)
   test_identical(nextget("url"), value)
   test_type("character", launch_remote(remote = remote_config(command = "echo", args = list(c("Test out:", ".", ">/dev/null")), rscript = "/usr/lib/R/bin/Rscript")))
   test_error(launch_remote(remote = ssh_config("ssh://127.0.0.1:5456", tunnel = TRUE)), "127.0.0.1")
-  test_zero(daemons(0L))
+  test_false(daemons(0L))
   Sys.sleep(1L)
-  test_zero(daemons(n = 2L, url = value <- "ws://:0", dispatcher = "none", remote = remote_config(quote = TRUE)))
+  test_true(daemons(n = 2L, url = value <- "ws://:0", dispatcher = "none", remote = remote_config(quote = TRUE)))
   test_true(status()$daemons != value)
-  test_zero(daemons(0L))
+  test_false(daemons(0L))
 }
 # mirai_map tests
 connection && {
@@ -147,15 +158,15 @@ connection && {
     if (is.null(tryCatch(mirai_map(list(1, "a", 2), sum)[.stop], error = function(e) NULL)))
       mirai_map(1:3, rnorm, .args = list(mean = 20, 2))[]
   })
-  test_true(!is_mirai_map(m))
+  test_false(is_mirai_map(m))
   test_type("list", m)
   test_equal(length(m), 3L)
   test_true(all(as.logical(lapply(m, is.numeric))))
   Sys.sleep(1L)
-  daemons(1, dispatcher = FALSE)
+  test_true(daemons(1, dispatcher = FALSE))
   mp <- mirai_map(list(x = "a"), function(...) do(...), do = function(x, y) sprintf("%s%s", x, y), .args = list("b"))
   test_print(mp)
-  test_identical(collect_mirai(mp)[["x"]], "ab")
+  test_identical(collect_mirai(mp, list(.flat = TRUE))[["x"]], "ab")
   test_identical(call_mirai(mp)[["x"]][["data"]], "ab")
   mres <- mirai_map(data.frame(1:3, 3:1), sum, .args = list(3L))[.flat]
   test_true(all(mres == 7L))
@@ -164,7 +175,7 @@ connection && {
   test_type("language", mirai_map(list(quote(1+2)), identity)[][[1]])
   test_class("Date", mirai_map(data.frame(x = as.Date("2020-01-01")), identity)[][[1]])
   test_true(is_mirai_error(mirai_map(1:2, function(x) daemons(1))[][[1]]))
-  test_zero(daemons(0L))
+  test_false(daemons(0L))
 }
 # parallel cluster tests
 library(parallel)
@@ -235,30 +246,29 @@ connection && {
 # advanced daemons and dispatcher tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
-  test_zero(daemons(url = "ws://:0", correctype = 0L, token = TRUE))
-  test_zero(daemons(0L))
+  test_true(daemons(url = "ws://:0", correctype = 0L, token = TRUE))
+  test_false(daemons(0L))
   test_zero(with(daemons(url = "tcp://:0", correcttype = c(1, 0), token = TRUE), {8L - 9L + 1L}))
-  test_zero(daemons(n = 2, "ws://:0"))
+  test_true(daemons(n = 2, "ws://:0"))
   test_type("integer", nextget("pid"))
   test_equal(length(nextget("url")), 1L)
   Sys.sleep(1L)
-  status <- daemons()
+  status <- status()
   test_type("list", status)
   test_zero(status[["connections"]])
   test_type("integer", status[["mirai"]])
   test_zero(status[["mirai"]][["awaiting"]])
   test_zero(status[["mirai"]][["executing"]])
   test_zero(status[["mirai"]][["completed"]])
-  test_zero(daemons(0))
-  test_equal(daemons(2, correcttype = NA), 2L)
-  test_equal(daemons()[["connections"]], 2L)
+  test_true(daemons(2, correcttype = NA))
+  test_equal(status()[["connections"]], 2L)
   test_type("list", res <- mirai_map(c(1,1), rnorm)[.progress])
-  test_true(res[[1L]] != res[[2L]])
-  test_equal(2L, daemons()[["connections"]])
+  test_type("double", res[[1L]])
+  test_type("double", res[[2L]])
   Sys.sleep(0.5)
-  test_zero(daemons(0L))
+  test_false(daemons(0L))
   Sys.sleep(1L)
-  test_zero(daemons(url = "tls+tcp://127.0.0.1:0", dispatcher = TRUE))
+  test_true(daemons(url = "tls+tcp://127.0.0.1:0", dispatcher = TRUE))
   test_type("character", launch_remote(remote = ssh_config(c("ssh://remotehost", "ssh://remotenode"), tunnel = TRUE, command = "echo")))
   test_equal(launch_local(), 1L)
   Sys.sleep(1L)
@@ -267,20 +277,18 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   m <- mirai("Seattle", .timeout = 1000)
   if (!is_error_value(m[])) test_equal(m[], "Seattle")
   test_class("errorValue", mirai(q(), .timeout = 1000)[])
-  test_zero(daemons(0))
-  Sys.sleep(0.5)
-  test_zero(daemons(n = 1L, url = local_url(), dispatcher = TRUE))
-  task <- mirai(TRUE)
+  test_true(daemons(n = 1L, url = local_url(), dispatcher = TRUE))
+  task <- mirai(substitute())
   url <- nextget("url")
-  test_equal(3L, daemon(url = url, maxtasks = 1L, cleanup = 0L, dispatcher = TRUE))
-  test_zero(daemons(n = 0L))
+  test_equal(daemon(url = url, maxtasks = 1L, cleanup = 0L, dispatcher = TRUE), 3L)
+  test_false(daemons(n = 0L))
   test_type("integer", .Random.seed)
 }
 # TLS tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
   cfg <- serial_config("custom", function(x) serialize(x, NULL), unserialize)
-  test_zero(daemons(url = host_url(), pass = "test", serial = cfg))
+  test_true(daemons(url = host_url(), pass = "test", serial = cfg))
   if (.Platform$OS.type == "unix") test_type("character", launch_remote(remote = cluster_config(command = "/bin/sh", options = "#SBATCH", rscript = file.path(R.home("bin"), "Rscript"))))
   test_equal(launch_local(1L), 1L)
   Sys.sleep(1L)
@@ -290,20 +298,23 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_zero(sum(collect_mirai(mm, ".flat")))
   m <- mirai(b, .timeout = 1000)
   if (!is_error_value(m[])) test_equal(m[], 2L)
-  test_zero(daemons(0))
+  test_false(daemons(0))
   test_tls <- function(cert) {
     file <- tempfile()
     on.exit(unlink(file))
     cat(cert[["server"]], file = file)
-    daemons(url = "tls+tcp://127.0.0.1:0", tls = file) == 0L && daemons(0L) == 0L
+    daemons(url = "tls+tcp://127.0.0.1:0", tls = file, tlscert = cert[["client"]]) &&
+      is.character(nextget("tls")) &&
+      grepl("cert.pem", launch_remote(tlscert = "cert.pem"), fixed = TRUE) &&
+      daemons(0L)
   }
-  test_true(test_tls(nanonext::write_cert(cn = "127.0.0.1")))
+  test_false(test_tls(nanonext::write_cert(cn = "127.0.0.1")))
 }
 # promises tests
 connection && requireNamespace("promises", quietly = TRUE) && Sys.getenv("NOT_CRAN") == "true" && {
   run_now <- getNamespace("later")[["run_now"]]
   Sys.sleep(0.5)
-  test_equal(daemons(1, notused = "wrongtype"), 1L)
+  test_true(daemons(1, notused = "wrongtype"))
   test_true(grepl("://", launch_remote(1L), fixed = TRUE))
   test_true(promises::is.promise(p1 <- promises::as.promise(mirai("completed"))))
   test_true(promises::is.promise(p2 <- promises::`%...>%`(mirai(Sys.sleep(0.1)), identity())))
@@ -325,13 +336,13 @@ connection && requireNamespace("promises", quietly = TRUE) && Sys.getenv("NOT_CR
   test_identical(names(mp[]), c("a", "b"))
   test_class("errorValue", mirai_map(1, function(x) stop(x), .promise = list(identity, identity))[][[1L]])
   run_now(1L)
-  test_zero(daemons(NULL))
+  test_false(daemons(NULL))
 }
 # mirai daemon limits tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
-  test_equal(daemons(1, cleanup = FALSE, maxtasks = 2L, id = 125L), 1L)
-  test_true(daemons_set())
+  test_true(daemons(1, cleanup = FALSE, maxtasks = 2L, id = 125L))
+  test_true(daemons_set("default"))
   test_equal(mirai(1)[], mirai(1)[])
   m <- mirai(0L)
   Sys.sleep(1L)
@@ -345,8 +356,9 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   res <- status()
   test_zero(res$connections)
   test_identical(res$events, c(129L, -129L))
-  test_zero(daemons(0))
-  test_equal(daemons(1, dispatcher = FALSE, maxtasks = 1L), 1L)
+  test_equal(info()[["cumulative"]], 2L)
+  test_false(daemons(0))
+  test_true(daemons(1, dispatcher = FALSE, maxtasks = 1L))
   test_zero(mirai(0L)[])
   Sys.sleep(0.5)
   test_zero(status()$connections)
@@ -354,13 +366,14 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_zero(mirai(0)[])
   Sys.sleep(1L)
   test_zero(status()$connections)
-  test_zero(daemons(0))
+  test_true(is.na(info()[["cumulative"]]))
+  test_false(daemons(0))
 }
 # mirai cancellation tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
   Sys.unsetenv("R_DEFAULT_PACKAGES")
-  test_equal(daemons(1, dispatcher = TRUE, cleanup = FALSE), 1L)
+  test_true(daemons(1, dispatcher = TRUE, cleanup = FALSE))
   m1 <- mirai({ Sys.sleep(1); res <<- "m1 done" })
   m2 <- mirai({ Sys.sleep(1); res <<- "m2 done" })
   Sys.sleep(0.1)
@@ -378,39 +391,98 @@ connection && Sys.getenv("NOT_CRAN") == "true" && {
   test_equal(status()$connections, 1L)
   test_equal(length(nextget("url")), 1L)
   test_class("miraiLaunchCmd", launch_remote(1))
-  test_zero(daemons(0))
+  test_false(daemons(0))
 }
 # additional stress testing
 connection && Sys.getenv("NOT_CRAN") == "true" && {
   Sys.sleep(0.5)
   q <- vector(mode = "list", length = 10000L)
   Sys.setenv(R_DEFAULT_PACKAGES = "stats,utils")
-  test_equal(daemons(4), 4L)
+  test_true(daemons(4))
   Sys.unsetenv("R_DEFAULT_PACKAGES")
-  test_type("list", everywhere(a <<- TRUE))
-  test_true(all(mirai_map(1:4, function(x) { Sys.sleep(0.2); a})[.flat]))
+  test_class("mirai_map", everywhere(a <<- FALSE))
+  test_true(all(everywhere(a <<- TRUE)[.flat]))
+  test_true(all(mirai_map(1:10, function(x) a)[.flat]))
   for (i in seq_len(10000L)) {q[[i]] <- mirai(1L); attr(q[[i]], "status") <- status()}
   test_equal(sum(unlist(collect_mirai(q))), 10000L)
   test_true(all(as.logical(lapply(lapply(q, attr, "status"), is.list))))
   for (i in seq_len(10000L)) {q[[i]] <- mirai({Sys.sleep(0.001); rnorm(1)}); attr(q[[i]], "status") <- status()}
   test_equal(length(unique(unlist(collect_mirai(q)))), 10000L)
   test_true(all(as.logical(lapply(lapply(q, attr, "status"), is.list))))
-  test_equal(daemons()[["mirai"]][["completed"]], 20008L)
-  test_zero(daemons(0))
+  test_equal(status()[["mirai"]][["completed"]], 20020L)
+  test_false(daemons(0))
 }
 # reproducible RNG tests
 connection && Sys.getenv("NOT_CRAN") == "true" && {
-  test_equal(2L, daemons(2, seed = 1234L))
-  test_equal(1L, launch_local())
+  test_true(daemons(2, seed = 1234L))
+  test_equal(launch_local(), 1L)
   test_type("character", launch_remote())
   Sys.sleep(0.5)
+  test_true(all(everywhere(TRUE)[.flat]))
   m <- mirai_map(1:12, rnorm)[]
-  test_zero(daemons(0))
+  test_false(daemons(0))
   Sys.sleep(0.5)
-  test_equal(4L, daemons(4, dispatcher = FALSE, seed = 1234L))
-  n <- mirai_map(1:12, rnorm)[]
-  test_zero(daemons(0))
+  test_true(daemons(4, dispatcher = "none", seed = 1234L, .compute = "gpu"))
+  with_daemons("gpu", {
+    test_true(all(everywhere(TRUE)[.flat]))
+    n <- mirai_map(1:12, rnorm)[]
+    test_false(daemons(NULL))
+  })
+  test_false(daemons_set("gpu"))
   test_identical(m, n)
 }
-test_zero(daemons(0))
+connection && requireNamespace("otel", quietly = TRUE) && Sys.getenv("NOT_CRAN") == "true" && {
+  record <- tryCatch(
+    getNamespace("otelsdk")$with_otel_record({
+      ns <- getNamespace("mirai")
+      unlockBinding("otel_tracing", ns)
+      ns[["otel_tracing"]] <- TRUE
+      lockBinding("otel_tracing", ns)
+      url <- local_url()
+      test_true(daemons(url = url, dispatcher = FALSE))
+      mp <- mirai_map(1:3, rnorm)
+      m1 <- mirai(stop("error"))
+      m2 <- mirai(getNamespace("rlang")$interrupt())
+      test_equal(daemon(url = url, maxtasks = 5L, dispatcher = FALSE), 3L)
+      test_true(is_error_value(m2[]))
+      test_false(daemons(0))
+    }),
+    error = function(cnd) NULL
+  )
+  length(record) && {
+    test_equal(length(record$traces), 13L)
+    test_equal(record$traces[[1L]]$name, "mirai::mirai")
+    test_equal(record$traces[[1L]]$kind, "client")
+    test_true(record$traces[[2L]]$attributes$mirai.id > record$traces[[1L]]$attributes$mirai.id)
+    test_equal(record$traces[[1L]]$parent, record$traces[[2L]]$parent)
+    test_equal(record$traces[[2L]]$links[[1L]]$span_id, record$traces[[3L]]$links[[1L]]$span_id)
+    test_equal(record$traces[[4L]]$name, "mirai::mirai_map")
+    test_equal(record$traces[[4L]]$kind, "internal")
+    test_equal(record$traces[[4L]]$links[[1L]]$span_id, record$traces[[3L]]$links[[1L]]$span_id)
+    test_equal(record$traces[[4L]]$span_id, record$traces[[2L]]$parent)
+    test_equal(record$traces[[5L]]$name, "mirai::mirai")
+    test_equal(record$traces[[5L]]$kind, "client")
+    test_equal(record$traces[[5L]]$parent, record$traces[[4L]]$parent)
+    test_equal(record$traces[[7L]]$name, "mirai::daemon->eval")
+    test_equal(record$traces[[7L]]$kind, "server")
+    test_equal(record$traces[[7L]]$parent, record$traces[[1L]]$span_id)
+    test_equal(record$traces[[10L]]$status, "error")
+    test_equal(record$traces[[10L]]$description, "miraiError")
+    test_equal(record$traces[[11L]]$status, "error")
+    test_equal(record$traces[[11L]]$description, "miraiInterrupt")
+    test_equal(record$traces[[12L]]$name, "mirai::daemon")
+    test_equal(record$traces[[12L]]$kind, "internal")
+    test_equal(record$traces[[12L]]$span_id, record$traces[[7L]]$links[[1L]]$span_id)
+    test_equal(record$traces[[13L]]$name, "mirai::daemons")
+    test_equal(record$traces[[13L]]$kind, "internal")
+    test_equal(record$traces[[13L]]$attributes$url, url)
+    test_equal(record$traces[[13L]]$attributes$n, 0L)
+    test_equal(record$traces[[13L]]$attributes$dispatcher, "false")
+    test_equal(record$traces[[13L]]$attributes$compute_profile, "default")
+    test_equal(record$traces[[13L]]$span_id, record$traces[[1L]]$links[[1L]]$span_id)
+    test_equal(record$traces[[13L]]$span_id, record$traces[[4L]]$links[[1L]]$span_id)
+    test_equal(record$traces[[13L]]$span_id, record$traces[[5L]]$links[[1L]]$span_id)
+  }
+}
+test_false(daemons(0))
 Sys.sleep(1L)
